/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2010, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"WriteMeshGmsh.h"

void
WriteMeshGmsh (FILE* file, char* dim, struct NODES Nodes,
    struct MESH Mesh0D, struct MESH Mesh1D, struct MESH Mesh2D,
    struct MESH Mesh3D, char* nset, int** NSets, char** NSetNames,
    struct PART Part)
{
  int i, j, elt_type0D, elt_type1D, elt_type2D, elt_type3D, eltnodeqty1D,
  eltnodeqty2D, eltnodeqty3D, nodeqty, eltnodeqty;
  int* node_print = ut_alloc_1d_int (Nodes.NodeQty + 1);

  eltnodeqty1D = neut_elt_nodeqty ("tri", Mesh1D.Dimension, Mesh1D.EltOrder);
  eltnodeqty2D = neut_elt_nodeqty ("tri", Mesh2D.Dimension, Mesh2D.EltOrder);
  eltnodeqty3D = neut_elt_nodeqty ("tri", Mesh3D.Dimension, Mesh3D.EltOrder);

  if (Mesh0D.EltQty != 0 && neut_gmsh_elt_nb ("tri", Mesh0D.Dimension,
	Mesh0D.EltOrder, &elt_type0D) != 0)
    ut_error_reportbug ();

  if (Mesh1D.EltQty != 0 && neut_gmsh_elt_nb ("tri", Mesh1D.Dimension,
	Mesh1D.EltOrder, &elt_type1D) != 0)
    ut_error_reportbug ();

  if (Mesh2D.EltQty != 0 && neut_gmsh_elt_nb ("tri", Mesh2D.Dimension,
	Mesh2D.EltOrder, &elt_type2D) != 0)
    ut_error_reportbug ();

  if (Mesh3D.EltQty != 0 && neut_gmsh_elt_nb ("tri", Mesh3D.Dimension,
	Mesh3D.EltOrder, &elt_type3D) != 0)
    ut_error_reportbug ();

/*----------------------------------------------------------------------
 * header */

  fprintf (file, "$MeshFormat\n");
  fprintf (file, "2 0 8\n");
  fprintf (file, "$Comments\n");
  fprintf (file, "contiguous\n");
  fprintf (file, "$EndComments\n");
  fprintf (file, "$EndMeshFormat\n");

/*----------------------------------------------------------------------
 * nodes */

  // looking for the nodes to print out (i.e. avoiding orphan nodes)
  // starting by higher dimension meshes since they can fill out
  // node_print first.
  nodeqty = 0;

  eltnodeqty = neut_elt_nodeqty ("tri", Mesh3D.Dimension, Mesh3D.EltOrder);
  for (i = 1; i <= Mesh3D.EltQty; i++)
    for (j = 0; j < eltnodeqty; j++)
      if (node_print[Mesh3D.EltNodes[i][j]] == 0)
      {
	nodeqty++;
	node_print[Mesh3D.EltNodes[i][j]] = 1;
      }
  if (nodeqty != Nodes.NodeQty)
  {
    eltnodeqty = neut_elt_nodeqty ("tri", Mesh2D.Dimension, Mesh2D.EltOrder);
    for (i = 1; i <= Mesh2D.EltQty; i++)
      for (j = 0; j < eltnodeqty; j++)
	if (node_print[Mesh2D.EltNodes[i][j]] == 0)
	{
	  nodeqty++;
	  node_print[Mesh2D.EltNodes[i][j]] = 1;
	}
  }
  if (nodeqty != Nodes.NodeQty)
  {
    eltnodeqty = neut_elt_nodeqty ("tri", Mesh1D.Dimension, Mesh1D.EltOrder);
    for (i = 1; i <= Mesh1D.EltQty; i++)
      for (j = 0; j < eltnodeqty; j++)
	if (node_print[Mesh1D.EltNodes[i][j]] == 0)
	{
	  nodeqty++;
	  node_print[Mesh1D.EltNodes[i][j]] = 1;
	}
  }

  if (nodeqty > Nodes.NodeQty)
    ut_error_reportbug ();

  fprintf (file, "$Nodes\n");
  fprintf (file, "%d\n", nodeqty);
  for (i = 1; i <= Nodes.NodeQty; i++)
    if (node_print[i] == 1)
    {
      fprintf (file, "%d ", i);
      ut_array_1d_fprintf (file, Nodes.NodeCoo[i], 3, "%.12f");
    }
  fprintf (file, "$EndNodes\n");

/*----------------------------------------------------------------------
 * elements */

  fprintf (file, "$Elements\n");
  int eltqty = 0;
  if (ut_string_inlist (dim, ',', "0"))
    eltqty += Mesh0D.EltQty;
  if (ut_string_inlist (dim, ',', "1"))
    eltqty += Mesh1D.EltQty;
  if (ut_string_inlist (dim, ',', "2"))
    eltqty += Mesh2D.EltQty;
  if (ut_string_inlist (dim, ',', "3"))
    eltqty += Mesh3D.EltQty;

  fprintf (file, "%d\n", eltqty);

  // 0D mesh
  if (ut_string_inlist (dim, ',', "0"))
  {
    if (Mesh0D.EltElset == NULL)
      neut_mesh_init_eltelset (&Mesh0D, NULL);
    for (i = 1; i <= Mesh0D.EltQty; i++)
    {
      fprintf (file, "%d %d 3 0 %d 0 ", i, elt_type0D, Mesh0D.EltElset[i]);
      ut_array_1d_int_fprintf (file, Mesh0D.EltNodes[i], 1, "%d");
    }
  }

  // 1D mesh
  if (ut_string_inlist (dim, ',', "1"))
  {
    if (Mesh1D.EltElset == NULL)
      neut_mesh_init_eltelset (&Mesh1D, NULL);

    for (i = 1; i <= Mesh1D.EltQty; i++)
    {
      fprintf (file, "%d %d 3 0 %d 0 ", i, elt_type1D, Mesh1D.EltElset[i]);
      ut_array_1d_int_fprintf (file, Mesh1D.EltNodes[i], eltnodeqty1D, "%d");
    }
  }

  // 2D mesh
  if (ut_string_inlist (dim, ',', "2"))
  {
    if (Mesh2D.EltElset == NULL)
      neut_mesh_init_eltelset (&Mesh2D, NULL);

    for (i = 1; i <= Mesh2D.EltQty; i++)
    {
      fprintf (file, "%d %d 3 0 %d 0 ", i, elt_type2D, Mesh2D.EltElset[i]);
      ut_array_1d_int_fprintf (file, Mesh2D.EltNodes[i], eltnodeqty2D, "%d");
    }
  }

  // 3D mesh
  if (ut_string_inlist (dim, ',', "3"))
  {
    if (Part.qty > 0)
      for (i = 1; i <= Mesh3D.EltQty; i++)
      {
	fprintf (file, "%d %d 3 0 %d %d ", i, elt_type3D, Mesh3D.EltElset[i], Part.elt_parts[i] + 1);
	ut_array_1d_int_fprintf (file, Mesh3D.EltNodes[i], eltnodeqty3D, "%d");
      }
    else
      for (i = 1; i <= Mesh3D.EltQty; i++)
      {
	fprintf (file, "%d %d 3 0 %d 0 ", i, elt_type3D, Mesh3D.EltElset[i]);
	ut_array_1d_int_fprintf (file, Mesh3D.EltNodes[i], eltnodeqty3D, "%d");
      }
  }

  fprintf (file, "$EndElements\n");

/*----------------------------------------------------------------------
 * node sets */

  if (nset != NULL)
  {
    fprintf (file, "\n/* node sets\n");
    neut_mesh_fprintf_gmsh_nsets (file, nset, NSets, NSetNames);
    fprintf (file, "\n*/\n");
  }
  
  ut_free_1d_int (node_print);

  return;
}
