/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2010, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"nefm_geo.h"

int
nefm_geo_face_interpolmesh (struct GEO Geo, int face, struct NODES* pN,
    struct MESH* pM)
{
  int i, tqty, ver, ver0;
  int verqty = Geo.FaceVerQty[face];
  double *p0 = NULL, *p1 = NULL, *p2 = NULL;
  int* nodes = ut_alloc_1d_int (3);
  int* elts = NULL;

  neut_nodes_set_zero (pN);
  neut_mesh_set_zero (pM);
  (*pM).Dimension = 2;
  (*pM).EltOrder = 1;

  for (i = 1; i <= Geo.FaceVerQty[face]; i++)
  {
    ver = Geo.FaceVerNb[face][i];
    neut_nodes_addnode (pN, Geo.VerCoo[ver], 1);
  }
  if (Geo.FacePt[face] == 0)
    neut_nodes_addnode (pN, Geo.FacePtCoo[face], 1);

  ver0 = (Geo.FacePt[face] != 0) ? Geo.FacePt[face] : Geo.FaceVerQty[face] + 1;
  p0 = Geo.VerCoo[ver0];

  tqty = 0;
  for (i = 1; i <= verqty; i++)
  {
    // if p0 is a face vertex: 
    // if the ver is p0 or the previous vertex, it is skipped
    if (i == Geo.FacePt[face]
	|| ut_num_rotpos (1, verqty, i, 1) == Geo.FacePt[face])
      continue;

    // recording coo of the 1st and 2nd pts
    nodes[0] = ver0;
    nodes[1] = i;
    nodes[2] = ut_num_rotpos (1, verqty, i, 1);
    p1 = Geo.VerCoo[nodes[1]];
    p2 = Geo.VerCoo[nodes[2]];

    neut_mesh_addelt (pM, nodes);

    // if the 3 points are aligned (on an edge of the domain), skipping.
    // use an angle-criterion instead.
    // 01/10: sounds that this never happens...
    if (ut_space_pointaligned (p1, p2, p0, 1e-6))
      continue;

    tqty++;
  }
  elts = ut_alloc_1d_int ((*pM).EltQty);
  for (i = 0; i < (*pM).EltQty; i++)
    elts[i] = i + 1;
  neut_mesh_addelset (pM, elts, (*pM).EltQty);
  
  ut_free_1d_int (elts);
  ut_free_1d_int (nodes);

  return tqty;
}


void
nefm_geo_face_1delts (struct GEO Geo, struct MESH Mesh1D, int
			     face, int** pelts, int* peltqty)
{
  int i, j, EdgeQty, edge, ori;

  (*peltqty) = 0;

  EdgeQty = Geo.FaceVerQty[face];
  for (i = 1; i <= EdgeQty; i++)
  {
    edge = Geo.FaceEdgeNb[face][i];
    (*peltqty) += Mesh1D.Elsets[edge][0];
  }
  
  (*pelts) = ut_alloc_1d_int (*peltqty);
  (*peltqty) = 0;

  for (i = 1; i <= EdgeQty; i++)
  {
    ori = Geo.FaceEdgeOri[face][i];
    edge = Geo.FaceEdgeNb[face][i];

    if (ori == 1)
      for (j = 1; j <= Mesh1D.Elsets[edge][0]; j++)
	(*pelts)[(*peltqty)++] = Mesh1D.Elsets[edge][j];
    else
      for (j = Mesh1D.Elsets[edge][0]; j >= 1; j--)
	(*pelts)[(*peltqty)++] = -Mesh1D.Elsets[edge][j];
  }

  return;
}
