/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2010, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"InputData_mm.h"

void
TreatArg_mm (int fargc, char **fargv, int argc, char **argv,
	     struct IN *pIn, struct DNEIGH *pDNeigh,
	     struct MAPMESH *pMapMesh)
{
  int initess = 0;
  int iniN = 0;

  char *body;
  int inibody = 0;

  SetDefaultOptions_mm (pIn, pDNeigh, pMapMesh);

  /* Reading options. */
  SetOptions_mm (pIn, pDNeigh, pMapMesh, &iniN, &initess, &body,
		 &inibody, fargc, fargv);
  SetOptions_mm (pIn, pDNeigh, pMapMesh, &iniN, &initess, &body,
		 &inibody, argc, argv);

  /* Checking the input options */
  if ((initess == 0 && iniN == 0) || (initess == 1 && iniN == 1))
    ut_arg_badarg ();		/* it's either initess==1 or iniN==1 */

  if (initess == 1)
  {				/* if tess file, the following */
    if ((*pDNeigh).ttype != 0	/* options should not be active. */
	|| (*pDNeigh).Id != -1 || (*pIn).nsite != 0 || (*pIn).ps2d == 1)
      ut_arg_badarg ();

    if (ut_file_exist ((*pIn).tess) == 0)	/* and the file must exist. */
      ut_file_open ((*pIn).tess, "r");

    (*pIn).input = 0;		/* input= tess file */
  }
  else
    (*pIn).input = 1;		/* input= N (,id) */

  if ((*pDNeigh).ttype != 2 && ((*pIn).nsitesize > 0 || (*pIn).nsite != 0))
  {
    ut_print_messagewnc (2, 72,
		"  `-nsiteXXX' options are forbidden with -ttype != 2. Maybe you should use -ttype 2?");
    abort ();
  }

  /* Writing file names */
  if (inibody == 0)
  {
    body = ut_alloc_1d_char (1000);

    if ((*pIn).input == 0)
      ut_string_body ((*pIn).tess, body);
    else if ((*pDNeigh).Id != -1)
      sprintf (body, "n%d-id%d", (*pDNeigh).NDensity, (*pDNeigh).Id);
    else
      sprintf (body, "n%d-rand", (*pDNeigh).NDensity);
  }

  (*pIn).msh = ut_string_addextension (body, ".msh");
  (*pIn).geof = ut_string_addextension (body, ".geof");
  (*pDNeigh).oin = ut_string_addextension (body, ".oin");
  (*pIn).nsitefile = ut_string_addextension (body, ".nsite");

  if (inibody == 0)
    ut_free_1d_char (body);

  return;
}
