/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2010, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"InputData_mm.h"

/* SetDefaultOptions set the options to their default values */
void
SetDefaultOptions_mm (struct IN *pIn, 
		      struct DNEIGH *pDNeigh, struct MAPMESH *pMapMesh)
{
  (*pIn).format = ut_alloc_1d_char (100);
  sprintf ((*pIn).format, "geof");

  (*pIn).geof = NULL;
  (*pIn).msh = NULL;

  (*pIn).verbosity = 0;
  (*pIn).input = -1;
  (*pIn).ps2d = 0;		/* like in module -T */
  (*pIn).nset = ut_alloc_1d_char (1000);
  (*pIn).faset = ut_alloc_1d_char (2);

  (*pIn).nsite = 0;
  (*pIn).nsitesize = -1;

  (*pDNeigh).ttype = 0;
  (*pDNeigh).NDensity = 100;
  (*pDNeigh).nN = 0;
  (*pDNeigh).Id = -1;

  (*pMapMesh).MeshSize = ut_alloc_1d_int (3);
  (*pMapMesh).MeshSize[0] = 20;
  (*pMapMesh).MeshSize[1] = 20;
  (*pMapMesh).MeshSize[2] = 20;
  (*pMapMesh).MeshOrder = 1;

  return;
}

void
SetOptions_mm (struct IN *pIn, struct DNEIGH *pDNeigh,
	       struct MAPMESH *pMapMesh, int *piniN, int *pinitess,
	       char **pbody, int *pinibody, int argc, char **argv)
{
  int i, ArgQty, Res;
  char **ArgList = ut_alloc_2d_char (101, 101);
  char *Arg = ut_alloc_1d_char (101);
  char* tessname = NULL;

  /* This is the possible argument list. */
  ArgQty = 0;
  sprintf (ArgList[++ArgQty], "-o");
  sprintf (ArgList[++ArgQty], "-n");
  sprintf (ArgList[++ArgQty], "-id");
  sprintf (ArgList[++ArgQty], "-ttype");
  sprintf (ArgList[++ArgQty], "-nset");
  sprintf (ArgList[++ArgQty], "-ps2d");
  sprintf (ArgList[++ArgQty], "-msize");
  sprintf (ArgList[++ArgQty], "-morder");
  sprintf (ArgList[++ArgQty], "-order");
  sprintf (ArgList[++ArgQty], "-v");
  sprintf (ArgList[++ArgQty], "-nsite");
  sprintf (ArgList[++ArgQty], "-nsitewidth");
  sprintf (ArgList[++ArgQty], "-faset");
  sprintf (ArgList[++ArgQty], "-msize3");
  sprintf (ArgList[++ArgQty], "-format");

  for (i = 1; i <= argc - 1; i++)
  {
    if (argv[i][0] != '-')
    {
      int n, id;
      tessname = ut_alloc_1d_char (strlen (argv[i]) + 1);
      sprintf (tessname, "%s", argv[i]);
      ut_string_fnrs (tessname, "n", " ", 1);
      ut_string_fnrs (tessname, "-id", "   ", 1);
      ut_string_fnrs (tessname, ".tess", "     ", 1);
      int status = sscanf (tessname, "%d%d", &n, &id);
      ut_free_1d_char (tessname);

      if (status != 2)
      {
	ut_print_message (2, 0, "failed\n");
	abort ();
      }
      else
      {
	(*pDNeigh).NDensity = n;
	(*pDNeigh).Id = id;
	(*piniN) = 1;
      }

      (*pinitess) = 0;

      /*
      if ((*pinitess) == 0)
      {
	(*pIn).tess = ut_alloc_1d_char (strlen (argv[i]) + 1);
	sprintf ((*pIn).tess, "%s", argv[i]);
	(*pinitess) = 1;
      }
      else
	ut_arg_badarg ();
      */
    }
    else
    {
      /* Searching option name (string completion stuff) */
      Res = ut_string_comp (argv[i], ArgList, ArgQty, Arg);
      if (Res == 1)
      {
	ut_print_lineheader (2);
	printf ("Several possibilities for option `%s'.\n", argv[i]);
	ut_arg_badarg ();
      }
      else if (Res == -1)
      {
	ut_print_lineheader (2);
	printf ("Unknown option `%s'.\n", argv[i]);
	ut_arg_badarg ();
      }

      if (strcmp (Arg, "-o") == 0 && i < argc - 1)
      {
	(*pbody) = ut_arg_nextaschar (argv, &i, Arg);
	ut_string_body (*pbody, *pbody);
	(*pinibody) = 1;
      }
      else if (strcmp (Arg, "-n") == 0 && i < argc - 1)
      {
	(*pDNeigh).NDensity = ut_arg_nextasint (argv, &i, Arg, 0, INT_MAX);
	(*piniN) = 1;
      }
      else if (strcmp (Arg, "-id") == 0 && i < argc - 1)
	(*pDNeigh).Id = ut_arg_nextasint (argv, &i, Arg, 0, INT_MAX);
      else if (strcmp (Arg, "-ttype") == 0 && i < argc - 1)
	(*pDNeigh).ttype = ut_arg_nextasint (argv, &i, Arg, 0, 2);
      else if (strcmp (Arg, "-format") == 0 && i < argc - 1)
	(*pIn).format = ut_arg_nextaschar (argv, &i, Arg);
      else if (strcmp (Arg, "-nset") == 0 && i < argc - 1)
	(*pIn).nset = ut_arg_nextaschar (argv, &i, Arg);
      else if (strcmp (Arg, "-faset") == 0 && i < argc - 1)
	(*pIn).faset = ut_arg_nextaschar (argv, &i, Arg);
      else if (strcmp (Arg, "-nsite") == 0 && i < argc - 1)
	(*pIn).nsite = ut_arg_nextasint (argv, &i, Arg, 0, 4);
      else if (strcmp (Arg, "-nsitewidth") == 0 && i < argc - 1)
	(*pIn).nsitesize = ut_arg_nextasreal (argv, &i, Arg, 0, DBL_MAX);
      else if (strcmp (Arg, "-ps2d") == 0 && i < argc - 1)
	(*pIn).ps2d = ut_arg_nextasint (argv, &i, Arg, 0, 1);
      else if (strcmp (Arg, "-msize") == 0 && i < argc - 1)
      {
	(*pMapMesh).MeshSize[0] = ut_arg_nextasint (argv, &i, Arg, 1, INT_MAX);
	ut_array_1d_int_set ((*pMapMesh).MeshSize, 3,
			     (*pMapMesh).MeshSize[0]);
      }
      else if (strcmp (Arg, "-msize3") == 0 && i < argc - 1)
      {
	(*pMapMesh).MeshSize[0] = ut_arg_nextasint (argv, &i, Arg, 1, INT_MAX);
	(*pMapMesh).MeshSize[1] = ut_arg_nextasint (argv, &i, Arg, 1, INT_MAX);
	(*pMapMesh).MeshSize[2] = ut_arg_nextasint (argv, &i, Arg, 1, INT_MAX);
      }
      else if (strcmp (Arg, "-morder") == 0 && i < argc - 1)
	(*pMapMesh).MeshOrder = ut_arg_nextasint (argv, &i, Arg, 1, 2);
      else if (strcmp (Arg, "-order") == 0 && i < argc - 1)
	(*pMapMesh).MeshOrder = ut_arg_nextasint (argv, &i, Arg, 1, 2);
      else if (strcmp (Arg, "-v") == 0 && i < argc - 1)
	(*pIn).verbosity = ut_arg_nextasint (argv, &i, Arg, 0, 2);
      else
      {
	ut_arg_badarg ();
	abort ();
      }
    }
  }

  ut_free_2d_char (ArgList, 101);
  ut_free_1d_char (Arg);


  return;
}
