/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2010, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "RMeshing.h"

void
RMeshing (struct NODES *pNodes, struct MESH *pMesh,
	  int ***pFoDNodes, struct IN In, struct DNEIGH *pDNeigh,
	  struct MAPMESH *pMap)
{
  int i, j, k, elt;
  int EltQty;
  struct GEO Geo;
  FILE *in;
  struct TESS Tess;
  char *expandnset = NULL;
  char *expandfaset = NULL;

  neut_utils_nset_expand (In.nset, &expandnset);
  neut_utils_nset_expand (In.faset, &expandfaset);

  /* Calculating element coordinates ------------------------------- */

  EltQty = (*pMap).MeshSize[0] * (*pMap).MeshSize[1] * (*pMap).MeshSize[2];

  (*pMap).EltCoo = gsl_matrix_float_calloc (EltQty + 1, 3);
  (*pMap).EltPoly = gsl_vector_int_calloc (EltQty + 1);
  (*pMap).EltId = (size_t *) calloc (EltQty + 1, sizeof (size_t));

  elt = 0;
  for (k = 1; k <= (*pMap).MeshSize[2]; k++)
    for (j = 1; j <= (*pMap).MeshSize[1]; j++)
      for (i = 1; i <= (*pMap).MeshSize[0]; i++)
      {
	elt++;
	gsl_matrix_float_set ((*pMap).EltCoo, elt, 0,
			      ((double) i -
			       0.5) / (double) (*pMap).MeshSize[0]);
	gsl_matrix_float_set ((*pMap).EltCoo, elt, 1,
			      ((double) j -
			       0.5) / (double) (*pMap).MeshSize[1]);
	gsl_matrix_float_set ((*pMap).EltCoo, elt, 2,
			      ((double) k -
			       0.5) / (double) (*pMap).MeshSize[2]);
      }

  /* Creating mapped mesh ------------------------------------------ */

  if (ut_string_inlist (In.format, ',', "geof"))
  {
    ut_print_message (0, 2, "Generating mesh ...\n");
    BuildMMesh3D ((*pMap).MeshSize, (*pMap).MeshOrder, pNodes, pMesh,
		  pFoDNodes);
  }

  /* Searching elsets ---------------------------------------------- */

  ut_print_message (0, 2, "Searching elsets ...\n");

  /* Calculation of EltPoly */
  if (In.input == 0)
  {
    in = ut_file_open (In.tess, "r");
    neut_tess_fscanf (in, &Tess);
    if (Tess.Id < 0)
    {
      ut_print_messagewnc (2, 72,
			   "Invalid tessellation file (must be generated with specified -n & -id with neper version >= 1.8.1.\n");
      abort ();
    }
    ut_file_close (in, In.tess, "r");

    Geo.DomainSize = ut_alloc_2d (3, 2);
    neut_tess_geodomainsize (Tess, &Geo);

    neut_nodes_scale (pNodes, Geo.DomainSize[0][1] - Geo.DomainSize[0][0],
		      Geo.DomainSize[1][1] - Geo.DomainSize[1][0],
		      Geo.DomainSize[2][1] - Geo.DomainSize[2][0]);
    neut_nodes_shift (pNodes, Geo.DomainSize[0][0],
		      Geo.DomainSize[1][0], Geo.DomainSize[2][0]);
    neut_mmesh_scale (pMap, Geo.DomainSize[0][1] - Geo.DomainSize[0][0],
		      Geo.DomainSize[1][1] - Geo.DomainSize[1][0],
		      Geo.DomainSize[2][1] - Geo.DomainSize[2][0]);
    neut_mmesh_shift (pMap, Geo.DomainSize[0][0],
		      Geo.DomainSize[1][0], Geo.DomainSize[2][0]);

    if (ut_string_inlist (In.format, ',', "geof"))
    {
      ut_print_message (0, 2, "Writing mapped mesh ...\n");
      WriteMeshMesh ((*pMap).MeshSize, (*pMap).MeshOrder, expandnset,
		     expandfaset, *pNodes, *pMesh, *pFoDNodes, In.geof);
    }

    RMeshingFTess (Tess, pMap, pDNeigh, In.verbosity);
  }
  else
  {
    if (ut_string_inlist (In.format, ',', "geof"))
    {
      ut_print_message (0, 2, "Writing mapped mesh ...\n");
      WriteMeshMesh ((*pMap).MeshSize, (*pMap).MeshOrder, expandnset,
		     expandfaset, *pNodes, *pMesh, *pFoDNodes, In.geof);
    }

    RMeshingFNId (In, pMap, pDNeigh);
  }

  /* Calculation of EltId */
  gsl_sort_int_index ((*pMap).EltId, (*pMap).EltPoly->data, 1, EltQty + 1);

  ut_free_1d_char (expandnset);
  ut_free_1d_char (expandfaset);

  return;
}
