/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2010, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"RMeshingFNId.h"

void
NIdSearchEltPoly (struct IN In, struct DNEIGH *pDNeigh, struct MAPMESH *pMap)
{
  int i, elt;
  int m = 0;
  int Germ;
  double dist;
  double *coo = ut_alloc_1d (3);
  gsl_vector *GermDist;

  int EltQty =
    (*pMap).MeshSize[0] * (*pMap).MeshSize[1] * (*pMap).MeshSize[2];

  GermDist = gsl_vector_alloc ((*pDNeigh).N + (*pDNeigh).nN + 1);

  gsl_vector_set (GermDist, 0, 1e30);	/* This is to avoid id 0 in the */
  /* minimum searching below.     */

  /* for every elt, the poly of which it belongs is searched out. */
  for (elt = 1; elt <= EltQty; elt++)
  {
    /* recording of the elt coordinates into coo */
    for (i = 0; i <= 2; i++)
      coo[i] = (double) gsl_matrix_float_get ((*pMap).EltCoo, elt, i);

    /* calculation of the distances bw the elt and the germs
     * (GermsCoo...+1 for memory skip)
     */
    for (i = 1; i <= (*pDNeigh).N; i++)
    {
      dist = ut_space_dist (coo, (*pDNeigh).GermsCoo[i] + 1);
      gsl_vector_set (GermDist, i, dist);
    }
    for (i = 1; i <= (*pDNeigh).nN; i++)
    {
      dist = ut_space_dist (coo, (*pDNeigh).nGermsCoo[i] + 1);
      gsl_vector_set (GermDist, (*pDNeigh).N + i, dist);
    }

    /* recording of the id of the nearest seed  */
    Germ = gsl_vector_min_index (GermDist);
    gsl_vector_int_set ((*pMap).EltPoly, elt, Germ);

    /* print percent evolution in case of verbosity */
    if (In.verbosity >= 1)
      PercentDisplay (elt, EltQty, 10, &m);
  }

  gsl_vector_free (GermDist);

  /* If the tessellation is periodic */
  /* applying GermtoGerm on EltPoly */
  if ((*pDNeigh).ttype == 1)
    for (elt = 1; elt <= EltQty; elt++)
    {
      Germ = gsl_vector_int_get ((*pMap).EltPoly, elt);

      if (Germ >= (*pDNeigh).N + 1)	/* modifying Germ */
      {
	Germ = (*pDNeigh).GermToGerm[Germ - (*pDNeigh).N];
	gsl_vector_int_set ((*pMap).EltPoly, elt, Germ);
      }
    }

  ut_free_1d (coo);

  return;
}
